/*
 * Copyright (C) 2022 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

module bram
#(
  parameter ADDR_WIDTH = 10,  //!< address bus width
  parameter DATA_WIDTH = 32   //!< data bus width
) (
  input  clk,
  input  [ADDR_WIDTH-1:0] addr,
  input  we,
  input  [DATA_WIDTH-1:0] wrdata,
  output [DATA_WIDTH-1:0] rddata
);

  localparam N = 2**ADDR_WIDTH;
  reg [DATA_WIDTH-1:0] memory_array[N-1:0];

  reg [DATA_WIDTH-1:0] r_rddata;
  //
  always @( posedge(clk) ) begin
    if ( we == 1'b1 ) begin
      memory_array[addr] <= wrdata;
    end
    else begin
      r_rddata <= memory_array[addr];
    end
  end
  //
  assign rddata = r_rddata;

endmodule

