/*
 * Copyright (C) 2022 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

module synthesis(
  input  clk,
  input  btn0,
  input  [3:0] sw,
  output [3:0] led
);

  // always reading and writting at the same address:
  wire [10:0] addr;
  assign addr[10:4] = 7'b1111111;
  assign addr[ 3:0] = 4'b0101;

  // write enable is a button:
  wire we;
  assign we = btn0;

  // write what is selected by the switches:
  wire [31:0] wrdata;
  assign wrdata[31:4] = 28'hFFFFFFF;
  assign wrdata[ 3:0] = sw;

  // show on the leds the 4 LSB of what is read:
  wire [31:0] rddata;
  assign led = rddata[3:0];

  wire [31:3] unused;
  assign unused = rddata[31:3];

  bram #(
    .ADDR_WIDTH( 11 ),
    .DATA_WIDTH( 32 )
  ) i_bram (
    .clk(    clk    ),
    .addr(   addr   ),
    .we(     we     ),
    .wrdata( wrdata ),
    .rddata( rddata )
  );

endmodule
