/*
 * Copyright (C) 2022 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <verilated.h>
#include "Vsynthesis.h"
#include "verilated_vcd_c.h"  // to trace in a vcd

int main(
  int argc,
  char** argv
) {
  uint64_t main_time = 0;     // current simulation time
  // This is a 64-bit integer to reduce wrap over issues and allow modulus.
  // This is in units of the timeprecision used in Verilog (or from
  // --timescale-override)

  // instantiation of the model:
  Vsynthesis*    theDut = new Vsynthesis;

  // to trace in a vcd:
  Verilated::traceEverOn(true);
  VerilatedVcdC* theVcd = new VerilatedVcdC;
  theDut->trace(theVcd, 99);  // trace 99 levels of hierarchy
  theVcd->open("output.vcd");

  // set some inputs:
  theDut->btn0 = 0;
  theDut->sw   = 0;
  //
  while ( !Verilated::gotFinish() ) { // triggered by a $finish in the verilog

    // toggle clock
    if ( (main_time % 10) == 1 ) {
      theDut->clk = 1;
    }
    if ( (main_time % 10) == 6 ) {
      theDut->clk = 0;
    }

    // toggle inputs:
    if ( main_time > 45 && main_time < 125 ) {
      theDut->btn0 = 1;
    } else {
      theDut->btn0 = 0;
    }
    //
    if ( main_time < 85 ) {
      theDut->sw = 0x0;
    } else {
      theDut->sw = 0xD;
    }

    theDut->eval();           // evaluate model
    theVcd->dump(main_time);  // dump in the vcd file
    main_time++;              // time passes...

    // finish:
    if ( main_time > 265 ) {
      break;
    }

  }

  theVcd->close(); // done tracing
  theDut->final(); // done simulating
  delete theDut;

  return 0;
}

